import json
import time
from concurrent import futures

import grpc

from eu.softfire.tub.entities.entities import ManagerEndpoint
from eu.softfire.tub.entities.repositories import save, find, delete
from eu.softfire.tub.exceptions.exceptions import ManagerNotFound, RpcFailedCall
from eu.softfire.tub.messaging.grpc import messages_pb2
from eu.softfire.tub.messaging.grpc import messages_pb2_grpc
from eu.softfire.tub.utils.utils import get_logger, get_config

logger = get_logger('eu.softfire.tub.messaging')
_ONE_DAY_IN_SECONDS = 60 * 60 * 24


def receive_forever():
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=config.getint('system', 'server_threads')))
    messages_pb2_grpc.add_RegistrationServiceServicer_to_server(RegistrationAgent(), server)
    binding = '[::]:%s' % get_config('messaging', 'bind_port', 50051)
    logger.info("Binding rpc registration server to: %s" % binding)
    server.add_insecure_port(binding)
    server.start()
    try:
        while True:
            time.sleep(_ONE_DAY_IN_SECONDS)
    except KeyboardInterrupt:
        server.stop(0)


class RegistrationAgent(messages_pb2_grpc.RegistrationServiceServicer):
    def unregister(self, request, context):
        logger.info("unregistering %s" % request)
        for manager_endpoint in find(ManagerEndpoint):
            if manager_endpoint.name == request.name:
                delete(manager_endpoint)
                return messages_pb2.ResponseMessage(result=0)
        return messages_pb2.ResponseMessage(result=1, error_message="manager endpoint not found")

    def register(self, request, context):
        logger.info("registering %s" % request)
        manager_endpoint = ManagerEndpoint()
        manager_endpoint.name = request.name
        manager_endpoint.endpoint = request.endpoint
        save(manager_endpoint)
        return messages_pb2.ResponseMessage(result=0)

    def __init__(self):
        self.stop = False

    def stop(self):
        self.stop = True